@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Callable, Iterable, get_type_hints
+from typing import TYPE_CHECKING, Any, Callable, get_type_hints
 
 try:
     import ipywidgets
@@ -16,6 +16,8 @@ from magicgui.types import Separator
 from magicgui.widgets import protocols
 
 if TYPE_CHECKING:
+    from collections.abc import Iterable
+
     from magicgui.widgets.bases import Widget
 
 
@@ -141,19 +143,6 @@ class _IPyWidget(protocols.WidgetProtocol):
         pass
 
 
-class EmptyWidget(_IPyWidget):
-    _ipywidget: ipywdg.Widget
-
-    def _mgui_get_value(self) -> Any:
-        raise NotImplementedError()
-
-    def _mgui_set_value(self, value: Any) -> None:
-        raise NotImplementedError()
-
-    def _mgui_bind_change_callback(self, callback: Callable):
-        pass
-
-
 class _IPyValueWidget(_IPyWidget, protocols.ValueWidgetProtocol):
     def _mgui_get_value(self) -> float:
         return self._ipywidget.value
