@@ -8,7 +8,7 @@ import warnings
 from contextlib import contextmanager, suppress
 from functools import partial
 from itertools import chain
-from typing import TYPE_CHECKING, Any, Callable, Iterable, Iterator, Sequence
+from typing import TYPE_CHECKING, Any, Callable
 
 import qtpy
 import superqt
@@ -31,6 +31,8 @@ from magicgui.widgets import Widget, protocols
 from magicgui.widgets._concrete import _LabeledWidget
 
 if TYPE_CHECKING:
+    from collections.abc import Iterable, Iterator, Sequence
+
     import numpy
 
 
@@ -230,23 +232,6 @@ class QBaseValueWidget(QBaseWidget, protocols.ValueWidgetProtocol):
         return value
 
 
-# BASE WIDGET
-
-
-class EmptyWidget(QBaseWidget):
-    def __init__(self, **kwargs: Any) -> None:
-        super().__init__(QtW.QWidget, **kwargs)
-
-    def _mgui_get_value(self) -> Any:
-        raise NotImplementedError()
-
-    def _mgui_set_value(self, value: Any) -> None:
-        raise NotImplementedError()
-
-    def _mgui_bind_change_callback(self, callback: Callable) -> None:
-        pass
-
-
 # STRING WIDGETS
 
 
