@@ -6,4 +6,4 @@ removed in future releases. Use at your own risk.
 
 from .schema._guiclass import button, guiclass, is_guiclass
 
-__all__ = ["guiclass", "button", "is_guiclass"]
+__all__ = ["button", "guiclass", "is_guiclass"]
