@@ -12,16 +12,17 @@ from __future__ import annotations
 import contextlib
 import warnings
 from dataclasses import Field, dataclass, field, is_dataclass
-from typing import TYPE_CHECKING, Any, Callable, ClassVar, Mapping, TypeVar, overload
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, TypeVar, overload
 
 from psygnal import SignalGroup, SignalInstance, evented
 from psygnal import __version__ as psygnal_version
 
 from magicgui.schema._ui_field import build_widget
 from magicgui.widgets import PushButton
-from magicgui.widgets.bases import ContainerWidget, ValueWidget
+from magicgui.widgets.bases import BaseValueWidget, ContainerWidget
 
 if TYPE_CHECKING:
+    from collections.abc import Mapping
     from typing import Protocol
 
     from typing_extensions import TypeGuard
@@ -36,7 +37,7 @@ if TYPE_CHECKING:
         def events(self) -> SignalGroup: ...
     # fmt: on
 
-__all__ = ["guiclass", "button", "is_guiclass", "unbind_gui_from_instance"]
+__all__ = ["button", "guiclass", "is_guiclass", "unbind_gui_from_instance"]
 _BUTTON_ATTR = "_magicgui_button"
 _GUICLASS_FLAG = "__magicgui_guiclass__"
 
@@ -130,13 +131,12 @@ def guiclass(
     >>> @guiclass
     ... class MyData:
     ...     x: int = 0
-    ...     y: str = 'hi'
+    ...     y: str = "hi"
     ...
     ...     @button
     ...     def reset(self):
     ...         self.x = 0
-    ...         self.y = 'hi'
-    ...
+    ...         self.y = "hi"
     >>> data = MyData()
     >>> data.gui.show()
     """
@@ -206,7 +206,7 @@ class GuiBuilder:
 
     def __get__(
         self, instance: object | None, owner: type
-    ) -> ContainerWidget[ValueWidget]:
+    ) -> ContainerWidget[BaseValueWidget]:
         wdg = build_widget(owner if instance is None else instance)
 
         # look for @button-decorated methods
@@ -317,7 +317,7 @@ def unbind_gui_from_instance(gui: ContainerWidget, instance: Any) -> None:
         An instance of a `guiclass`.
     """
     for widget in gui:
-        if isinstance(widget, ValueWidget):
+        if isinstance(widget, BaseValueWidget):
             widget.changed.disconnect_setattr(instance, widget.name, missing_ok=True)
 
 
