@@ -42,11 +42,10 @@ example_files = [
     str(f) for f in EXAMPLES.rglob("*.py") if all(x not in str(f) for x in EXCLUDED)
 ]
 
-# if os is Linux and python version is 3.9 and backend is PyQt5
+# if os is Linux and backend is PyQt5
 LINUX = sys.platform.startswith("linux")
-PY39 = sys.version_info >= (3, 9)
 PYQT5 = "PyQt5" in sys.modules
-if LINUX and PY39 and PYQT5:
+if LINUX and PYQT5:
     # skip range_slider example because of superqt c++ wrapped item bug
     example_files = [f for f in example_files if "range_slider" not in f]
 
@@ -70,5 +69,5 @@ def test_examples(fname, monkeypatch):
             pytest.skip("numpy unavailable: skipping image example")
     finally:
         if "waveform" in fname:
-            type_map._type_map._TYPE_DEFS.pop(int, None)
-            type_map._type_map._TYPE_DEFS.pop(float, None)
+            type_map.TypeMap.global_instance()._type_defs.pop(int, None)
+            type_map.TypeMap.global_instance()._type_defs.pop(float, None)
